import edu.smcvt.graphics.*;
import java.awt.*;
import java.util.*;

public class FireworksWorld extends AnimationWorld{

	int count;
	Color skyColor;
	Color grassColor;	
	Random r;
		
	static final int COUNT_MAX = 256;
	static final double DARK_ENOUGH = 0.65;
	static final double SKY_PROPORTION = 0.7;
	static final int MAX_FIREWORKS = 10;
	public final double G = -0.0001; 
	public final double WIND_SPEED = 0.0000005;
	public static final double RADIUS_SCALAR = 20.;
	
	public void resetWorld(){
		super.resetWorld();
		count = 1;
		calcColors();
	}
		
	
	public FireworksWorld(){
		super();
		resetWorld();
		r = new Random();
		setName("Fireworks World");
	}
	
	public void calcColors(){
		int sr,sg,sb;
		int gr,gg,gb;
		
		int skyIndex = Math.min(count,COUNT_MAX);
		
		sr = 256 - skyIndex;
		sg = 160 - skyIndex/2;
		sb = 160 - skyIndex/2;
		
		gr = 32 * sr / 256;
		gg = 256 * sg / 256;
		gb = 128 * sb / 256;
		
		
		skyColor = new Color(sr,sg,sb);
		grassColor = new Color(gr,gg,gb);
	}
	
	public void worldTick(){
		count = (count + 1 > count) ? count + 1 : COUNT_MAX;
		calcColors();
		
		if((((double)count)/COUNT_MAX > DARK_ENOUGH) && (size() < MAX_FIREWORKS)){
			boolean added = false;
			AnimObject ao = null;
			double shouldAddSomething = r.nextDouble();
			if(shouldAddSomething < 0.00625){
				ao = new SimpleFirework(this,r.nextInt(15)+40,r.nextInt(10)+50,10+r.nextInt(15),r.nextDouble()/(RADIUS_SCALAR),(8.+r.nextInt(2))/(10.+r.nextInt(2)),0.01,0.5+.0625*(r.nextInt(5)-2),r.nextDouble()*Math.PI/4 - Math.PI / 8,new Color(r.nextInt(256),r.nextInt(256),r.nextInt(256)));
			} else if(shouldAddSomething < 0.075){
				ao = new SimpleSparklingFirework(this,r.nextInt(15)+40,r.nextInt(10)+50,10+r.nextInt(15),r.nextDouble()/(RADIUS_SCALAR),(8.+r.nextInt(2))/(10.+r.nextInt(2)),0.01,0.5+.0625*(r.nextInt(5)-2),r.nextDouble()*Math.PI/4 - Math.PI / 8,new Color(r.nextInt(256),r.nextInt(256),r.nextInt(256)));
			}
			if(ao != null){
				added = add(ao); 
				if(!added) System.out.println("oops");
			}
		}
	}
	
	public int calcHorizon(Dimension d){ return (int)(d.height * SKY_PROPORTION); }

	public void paintEnviron(Graphics g, Dimension curSize){
		int skyHeight = calcHorizon(curSize);
		int grassHeight = curSize.height - skyHeight;
	
		g.setColor(skyColor);
		g.fillRect(0,0,curSize.width,skyHeight);
		g.setColor(grassColor);
		g.fillRect(0,skyHeight,curSize.width,grassHeight);
		g.setColor(Color.BLACK);
		g.drawLine(0,skyHeight,curSize.width,skyHeight);
	}

}